:root {
  --md-accent-fg-color: #727272;
  --md-accent-font-color: #727272;
}

@media only screen and (min-width: 76.25em) {
  .md-main__inner {
    max-width: none;
  }
  .md-sidebar--primary {
    left: 0;
  }
  .md-sidebar--secondary {
    right: 0;
    margin-left: 0;
    -webkit-transform: none;
    transform: none;   
  }
}

/*.md-header__button {
    padding:  0px; 
}
*/
.md-header__source {
 }

.md-typeset h1 {
    line-height:  1;
    margin: 0px;
}

h2 {
    color:  var(--md-accent-font-color);
    border-top:  var(--md-accent-font-color) thin solid;
    border-bottom:  var(--md-accent-font-color) thin solid;
}

.blog-post-preview {
    border-left:  var(--md-accent-font-color) thin solid;
    padding-left:  10px;
    margin-top:  10px;
    margin-bottom: 24px;
    font-size: 0.9em;
}

aside.blog-post-aside {
    /*border-top:  var(--md-accent-font-color) thin solid;*/
    /*margin-top:  24px;*/
    /*padding-top:  12px;*/
}

div.contact_spacer {
    display:  block;
    height:  20px;
}

h3.title_news {
    margin-top:  10px;
    margin-bottom:  0px;
}

div.tabbed-set {
    background-color:  #fafafa;
}

.tabbed-content {
    padding:  5px;
}

li li {
    list-style-type: circle;
}

li li li {
    list-style-type: "–";
    font-size: 0.9em;
}

.mermaid {
  display: flex;
  justify-content: center;
}

/* Width manipulations etc. */

.md-grid {
  max-width: 1024px; 
}

@media (max-width: 891.98px) {
    .navbar-collapse.show { /* csslint allow: adjoining-classes */
        overflow-y: auto;
        max-height: calc(100vh - 3.5rem);
    }
}

.dropdown-item.open { /* csslint allow: adjoining-classes */
    color: #fff;
    background-color: #2FA4E7;
}

.dropdown-submenu > .dropdown-menu {
    margin: 0 0 0 1.5rem;
    padding: 0;
    border-width: 0;
}

.dropdown-submenu > a::after {
    display: block;
    content: " ";
    float: right;
    width: 0;
    height: 0;
    border-color: transparent;
    border-style: solid;
    border-width: 5px 0 5px 5px;
    border-left-color: #ccc;
    margin-top: 5px;
    margin-right: -10px;
}

.dropdown-submenu:hover > a::after {
    border-left-color: #fff;
}

@media (min-width: 892px) {
    .dropdown-menu {
        overflow-y: auto;
        max-height: calc(100vh - 3.5rem);
    }

    .dropdown-submenu {
        position: relative;
    }

    .dropdown-submenu > .dropdown-menu {
        /* csslint ignore:start */
        position: fixed !important;
        /* csslint ignore:end */
        margin-top: -9px;
        margin-left: -2px;
        border-width: 1px;
        padding: 0.5rem 0;
    }

    .dropdown-submenu.pull-left { /* csslint allow: adjoining-classes */
        float: none;
    }

    .dropdown-submenu.pull-left > .dropdown-menu { /* csslint allow: adjoining-classes */
        left: -100%;
        margin-left: 10px;
    }
}
